@@ -96,6 +96,17 @@ class ConnectionTimeoutException(KempTechApiException):
             msg, is_xml_msg=False, code=code)
 
 
+class RateLimitError(KempTechApiException):
+    def __init__(self):
+        msg = (
+            "Request rejected due to API rate limiting"
+        )
+        # Include HTTP429 in the error code,
+        # to indicate that a Too Many Requests error occurred
+        super(RateLimitError, self).__init__(
+            msg, is_xml_msg=False, code=429)
+
+
 def get_parameter_message(obj, parameters):
     try:
         param = parameters['param']
