@@ -1,5 +1,6 @@
 import json
 import logging
+from time import sleep
 
 import requests
 from requests import exceptions, Session
@@ -13,7 +14,8 @@ from .exceptions import (
     KempTechApiException,
     ConnectionTimeoutException,
     GenericObjectMissingLoadMasterInfo,
-    UnauthorizedAccessError)
+    UnauthorizedAccessError,
+    RateLimitError)
 from .utils import UseTlsAdapter, send_response, HTTPBasicAuthUTF8
 
 
@@ -21,6 +23,12 @@ requests.packages.urllib3.disable_warnings()
 log = logging.getLogger(__name__)
 logging.basicConfig()
 
+DEFAULT_RETRY_LIMIT = 3
+RETRY_DELAY_INITIAL = 0.5
+RETRY_DELAY_GROWTH_FACTOR = 2
+
+def get_retry_delay(attempt_number):
+    return RETRY_DELAY_INITIAL * RETRY_DELAY_GROWTH_FACTOR ** attempt_number
 
 class HttpClient(object):
     """Client that performs HTTP requests."""
@@ -29,7 +37,7 @@ class HttpClient(object):
     endpoint = None
 
     def __init__(self, tls_version=utils.DEFAULT_TLS_VERSION, cert=None,
-                 user=None, password=None, auth_handler=None):
+                 user=None, password=None, auth_handler=None, max_retries=DEFAULT_RETRY_LIMIT):
         self.cert = cert
         self.auth = (user, password)
         self._auth_handler = auth_handler
@@ -37,7 +45,7 @@ class HttpClient(object):
         self._tls_session = Session()
         self._tls_session.mount("http://", UseTlsAdapter(self._tls_version))
 
-        self._retry_count = 0
+        self._retry_count = max_retries
 
     def __enter__(self):
         return self
@@ -119,17 +127,27 @@ class HttpClient(object):
             raise ConnectionTimeoutException(self.ip_address)
         except (exceptions.ReadTimeout, exceptions.ConnectionError) as e:
             if retries < self._retry_count:
-                log.warning("A %s occurred to %s.", e.__class__.__name__,
-                            self.ip_address)
-                return self._do_request(http_method,
-                                        rest_command,
-                                        parameters,
-                                        file,
-                                        retries + 1)
+                stepback_time = get_retry_delay(retries)
+                log.debug(
+                    "A %s occurred to %s, retrying in %.2f seconds.",
+                    e.__class__.__name__,
+                    self.ip_address,
+                    stepback_time)
+                sleep(stepback_time)
+                return self._do_request(
+                    http_method, rest_command,
+                    parameters=parameters, file=file, data=data,
+                    headers=headers, retries=retries+1)
             else:
                 log.warning("A repeated %s occurred to %s.",
                             e.__class__.__name__, self.ip_address)
-                raise
+                # When the LM rate limits WUI/API connections,
+                # it abruptly closes the connection
+                # leading to SSLError: EOF occurred in violation of protocol
+                if isinstance(e, exceptions.SSLError) and 'EOF' in str(e):
+                    raise RateLimitError
+                else:
+                    raise
         except exceptions.URLRequired:
             log.error("%s is an invalid URL", cmd_url)
             raise
